@@ -33,6 +33,8 @@ from typing import Any, Dict, Iterable, List, Mapping, Optional, Set, Union
 import packaging.requirements
 import packaging.version
 
+from redgrease.utils import safe_str
+
 default_redgrease = packaging.requirements.Requirement("redgrease[runtime]")
 """The default redgrease runtime package. Latest from PyPi."""
 
@@ -50,6 +52,53 @@ PackageOption = Optional[Union[bool, str, Version, Requirement]]
 """Type alias for valid types of RedGrease package version specification."""
 
 
+def safe_requirement(req: Any) -> Requirement:
+    """Parse a Requirement safely
+
+    Args:
+        req (Any):
+            Value to parse
+
+    Returns:
+        Requirement:
+            Parsed Requirement object
+
+    Raises:
+        packaging.requirements.InvalidRequirement
+            If input could not be parsed.
+    """
+    if not isinstance(req, Requirement):
+        req = Requirement(safe_str(req))
+    return req
+
+
+def same_name(req1: Any, req2: Any) -> bool:
+    """Check if two requirements have the same name.
+
+    Version numbers, extras etc is ignored.
+
+    This means that `foo>1.2.3` will match `foo[bar]==0.1.2`,
+    because both are from the same `foo` package.
+
+
+    Args:
+        req1 (Any):
+            First requirement to parse.
+
+        req2 (Any):
+            Second requirement to parse.
+
+    Returns:
+        bool:
+            `True` if the two requirements share the same name.
+            `False` if they don't share name, or if there was a parse error.
+    """
+    try:
+        return safe_requirement(req1).name == safe_requirement(req2).name
+    except packaging.requirements.InvalidRequirement:
+        return False
+
+
 def read_requirements(
     *requirements_file_paths: str,
 ) -> Set[Requirement]:
@@ -140,7 +189,7 @@ def from_packages_dict(
 
 def resolve_requirements(
     requirements: Iterable[Union[str, Requirement]],
-    enforce_redgrease: PackageOption = False,
+    enforce_redgrease: PackageOption = None,
 ) -> Set[Requirement]:
     """Attempts to resolve the given requirements, and that the desired version of
     redgrease is, or is NOT, included.
@@ -156,13 +205,13 @@ def resolve_requirements(
 
             It can take several optional types::
 
-                - None : enforces that 'redgrease' is NOT in thequirements,
-                    any redgrease requirements will be removed.
+                - None : no enforcing. requirements are only deduplicated, with or
+                    without 'redgrease'
 
                 - bool :
                     True, enforces latest 'redgrease' package on PyPi,
-                    False, no enforcing. requirements are passed as-is,
-                        with or without 'redgrease'
+                    False, enforces that 'redgrease' is NOT in the quirements set,
+                        any redgrease requirements will be removed.
 
                 - str :
                     a. Specific version. E.g. "1.2.3".
@@ -177,7 +226,7 @@ def resolve_requirements(
 
                 - Requirment : behaves just as string version (d.)
 
-            Defaults to False.
+            Defaults to None.
 
     Returns:
         Set[Requirement]:
@@ -187,26 +236,31 @@ def resolve_requirements(
         InvalidRequirement:
             If any of the requirements are not well-formed.
     """
-    requirements_set = set(
-        [packaging.requirements.Requirement(str(req)) for req in requirements]
+    # Parse requirments and, remove duplicates
+    requirements_set = (
+        {packaging.requirements.Requirement(str(req)) for req in set(requirements)}
         if requirements
-        else []
+        else set()
     )
 
+    if enforce_redgrease is None:
+        # If there is no requirement to keep nor remove redgrease,
+        # then the requirments are returned, as-is
+        return requirements_set
+
     packages = to_packages_dict(requirements_set)
 
-    if enforce_redgrease is None:
-        # enforce_redgrease is explicitly None, meaning we enforce that
-        # redgrease should NOT be in ther requirements set
-        # so we remove it
-        if "redgrease" in packages:
-            del packages["redgrease"]
-        return from_packages_dict(packages)
-
     if isinstance(enforce_redgrease, bool):
-        if enforce_redgrease and "redgrease" not in packages:
+        if not enforce_redgrease:
+            # enforce_redgrease is explicitly False, meaning we enforce that
+            # redgrease should NOT be in ther requirements set
+            # so we remove it
+            packages.pop("redgrease", None)
+            return from_packages_dict(packages)
+
+        if "redgrease" not in packages:
             # enforce_redgrease is True  (i.e: any version of redgrease)
-            # and it is not currently in the requirement set
+            # but it is not currently specified in the requirement set
             # so we add the latest version from PyPi,
             packages["redgrease"] = [default_redgrease]
         return from_packages_dict(packages)
